@@ -3,8 +3,8 @@
 potentiostat
 ============
 
-    This module implements the serial interface to the Rodeostat open source 
-    Potentiostat Shield.
+    This module implements the serial interface to the Rodeostat open source
+    Potentiostat.
 
     :copyright: (c) 2017 by IO Rodeo 
     :license: MIT, see LICENSE.txt for more details.
@@ -134,26 +134,30 @@ CurrRange1uA = '1uA'
 CurrRange10uA = '10uA'
 CurrRange100uA = '100uA'
 CurrRange1000uA = '1000uA'
+CurrRange10000uA = '10000uA'
 CurrRange12000uA = '12000uA'
 CurrRange24000uA = '24000uA'
 
 CurrRangeListNanoAmp = [CurrRange1uA, CurrRange10uA, CurrRange100nA, CurrRange60nA]
 CurrRangeListMicroAmp = [CurrRange1uA, CurrRange10uA, CurrRange100uA, CurrRange1000uA]
-CurrRangeListMilliAmp = [CurrRange100uA, CurrRange1000uA, CurrRange12000uA, CurrRange24000uA]
+CurrRangeList10MilliAmp = [CurrRange10uA, CurrRange100uA, CurrRange1000uA, CurrRange10000uA]
+CurrRangeList24MilliAmp = [CurrRange100uA, CurrRange1000uA, CurrRange12000uA, CurrRange24000uA]
 
 HwVariantToCurrRangesDict = {
         'nanoAmpV0.1'         :  CurrRangeListNanoAmp,
         'microAmpV0.1'        :  CurrRangeListMicroAmp, 
-        'milliAmpV0.1'        :  CurrRangeListMilliAmp,
+        'milliAmpV0.1'        :  CurrRangeList24MilliAmp,
         'AD8250_nanoAmpV0.1'  :  CurrRangeListNanoAmp,
         'AD8250_microAmpV0.1' :  CurrRangeListMicroAmp, 
-        'AD8250_milliAmpV0.1' :  CurrRangeListMilliAmp,
+        'AD8250_milliAmpV0.1' :  CurrRangeList24MilliAmp,
         'AD8251_nanoAmpV0.1'  :  CurrRangeListNanoAmp,
         'AD8251_microAmpV0.1' :  CurrRangeListMicroAmp, 
-        'AD8251_milliAmpV0.1' :  CurrRangeListMilliAmp,
+        'AD8251_milliAmpV0.1' :  CurrRangeList24MilliAmp,
         '10V_nanoAmpV0.2'     :  CurrRangeListNanoAmp,
         '10V_microAmpV0.2'    :  CurrRangeListMicroAmp, 
-        '10V_milliAmpV0.2'    :  CurrRangeListMilliAmp,
+        '10V_milliAmpV0.2'    :  CurrRangeList24MilliAmp,
+        '10V_10MilliAmpV0.2'  :  CurrRangeList10MilliAmp,
+        '10V_24MilliAmpV0.2'  :  CurrRangeList24MilliAmp,
         }
 
 TimeUnitToScale = {'s': 1.e-3, 'ms': 1}
@@ -165,21 +169,21 @@ PklOutputFileType = 1
 
 class Potentiostat(serial.Serial):
 
-    """Provides a high level interface  performing serial communications with the Rodeostat. 
+    """Provides a high level interface for serial communications with the Rodeostat. 
 
-    :param str port: serial port associated with the Rodeostat, e.g. /dev/ttyACM0, COM1, etc.
-
-    :Keyword Arguments:
+    Args:
+        port (str): serial port associated with the Rodeostat, e.g. /dev/ttyACM0, COM1, etc.
 
+    Keyword Args:
+        timeout (float): serial port ready timeout (seconds)
 
     """
+
     
     ResetSleepDt = 0.0
     Baudrate = 115200
 
-    def __init__(self, port, timeout=10.0, debug=False):
-        """ Constructor
-        """
+    def __init__(self, port, timeout=10.0):
         params = {'baudrate': self.Baudrate, 'timeout': timeout}
         super(Potentiostat,self).__init__(port,**params)
         time.sleep(self.ResetSleepDt)
@@ -194,6 +198,10 @@ class Potentiostat(serial.Serial):
 
     def get_hardware_variant(self):
         """Returns a string representing the hardware variant.
+
+        Returns:
+            str: hardware variant, e.g. '10V_microAmpV0.2'
+
         """
         cmd_dict = {CommandKey: GetVariantCmd}
         msg_dict = self.send_cmd(cmd_dict)
@@ -201,15 +209,17 @@ class Potentiostat(serial.Serial):
 
 
     def stop_test(self):
-        """Stops (any) currently running tests.
-
-        """
+        """Stops (any) currently running tests. """
         cmd_dict = {CommandKey: StopTestCmd}
         msg_dict = self.send_cmd(cmd_dict)
 
 
     def get_volt(self):
-        """Returns the output voltage setting, i.e, the potential between the working and reference electrode. 
+        """Returns the output voltage setting, i.e, the potential between the
+        working and reference electrode. 
+
+        Returns:
+            float: output voltage setting, units (V)
 
         """
         cmd_dict = {CommandKey: GetVoltCmd}
@@ -219,7 +229,14 @@ class Potentiostat(serial.Serial):
 
 
     def set_volt(self,volt):
-        """Sets the output voltage setting, i.e., the potential between the working and reference electrode. 
+        """Sets the output voltage setting, i.e., the potential between the
+        working and reference electrode. 
+
+        Args:
+            volt (float): output voltage setting, units (V)
+
+        Returns:
+            float: output voltage setting, units (V)
 
         """
         cmd_dict = {CommandKey: SetVoltCmd, VoltKey: volt}
@@ -229,7 +246,10 @@ class Potentiostat(serial.Serial):
 
 
     def get_curr(self):
-        """Returns and immediate measurement of electrical current flowing in/out of the working electrode. 
+        """Gets an immediate measurement of the working electrode current. 
+
+        Returns:
+            float: working electrode current, units (uA)
 
         """
         cmd_dict = {CommandKey: GetCurrCmd}
@@ -239,7 +259,11 @@ class Potentiostat(serial.Serial):
 
 
     def get_ref_volt(self):
-        """Returns the measurement of the potential between the working and reference electrode. 
+        """Gets an immediate measurement of the potential between the working
+        and reference electrode. 
+
+        Returns:
+            float: potential between working and reference electrode, units (V)
 
         """
         cmd_dict = {CommandKey: GetRefVoltCmd}
@@ -249,7 +273,14 @@ class Potentiostat(serial.Serial):
 
 
     def get_param(self,testname):
-        """Returns the current values of the parameters for the specified voltammetric test. 
+        """Returns the current values of the parameters for the specified
+        voltammetric test. 
+
+        Args:
+            testname (str): name of voltammetric test, e.g., 'cyclic'
+
+        Returns:
+            dict: current values of test parameters 
 
         """
         cmd_dict = {CommandKey: GetParamCmd, TestKey: testname} 
@@ -260,6 +291,13 @@ class Potentiostat(serial.Serial):
     def set_param(self,testname,param):
         """Sets the parameters for the specified voltammetric test.
 
+        Args:
+            testname (str): name of voltammetric test
+            param   (dict): desired test parameters 
+
+        Returns:
+            dict: test parameters for testname (from device)
+
         """
         cmd_dict = {CommandKey: SetParamCmd, TestKey: testname, ParamKey: param}
         msg_dict = self.send_cmd(cmd_dict)
@@ -267,7 +305,14 @@ class Potentiostat(serial.Serial):
 
 
     def set_volt_range(self,volt_range):
-        """Sets the output voltage range (V)- used when setting output voltage manually.
+        """Sets the output voltage range (V)- used when setting output voltage
+        manually.
+
+        Args:
+            volt_range (str): desired output voltage range, e.g., '1V'
+
+        Returns:
+            dict: voltage range (from device)
 
         """
         if not volt_range in self.get_all_volt_range():
@@ -280,6 +325,9 @@ class Potentiostat(serial.Serial):
     def get_volt_range(self):
         """Gets the current value for the output voltage range (V).
 
+        Returns:
+            dict: voltage range (from device)
+
         """
         cmd_dict = {CommandKey: GetVoltRangeCmd}
         msg_dict = self.send_cmd(cmd_dict)
@@ -287,14 +335,23 @@ class Potentiostat(serial.Serial):
 
 
     def get_all_volt_range(self):
-        """Gets a list of voltage ranges supported by the device.
+        """Gets a list of the voltage ranges supported by the device.
+
+        Returns:
+            list: available voltage ranges, e.g., ['1V', '2V', '5V', '10V']
 
         """
         return HwVariantToVoltRangesDict[self.hardware_variant]
 
 
     def set_curr_range(self,curr_range):
-        """Sets the measurement current range (uA).
+        """Sets the range setting for measuring working electrode (uA).
+
+        Args:
+            curr_range (str): the desired workign electrode current range setting, e.g., '10uA'
+
+        Returns:
+            str: working electrode current range setting (from device), e.g. '10uA'
 
         """
         if not curr_range in self.get_all_curr_range(): 
@@ -305,7 +362,10 @@ class Potentiostat(serial.Serial):
 
 
     def get_curr_range(self):
-        """Gets the current value of the measurement current range (uA).
+        """Gets the device's range setting for measuring working electrode current (uA).
+
+        Returns:
+            str : working electrode current range setting (from device), e.g., '10uA'
 
         """
         cmd_dict = {CommandKey: GetCurrRangeCmd}
@@ -316,6 +376,8 @@ class Potentiostat(serial.Serial):
     def get_all_curr_range(self):
         """Gets a list of all current ranges supported by the device.
 
+        Returns:
+            list: all available current ranges as str e.g., ['1uA', 10uA', '100uA', '1000uA']
         """
         return HwVariantToCurrRangesDict[self.hardware_variant]
 
@@ -323,6 +385,8 @@ class Potentiostat(serial.Serial):
     def get_device_id(self):
         """Gets the current value of the device identification number
 
+        Returns:
+            int: device identification number
         """
         cmd_dict = {CommandKey: GetDeviceIdCmd}
         msg_dict = self.send_cmd(cmd_dict)
@@ -332,6 +396,12 @@ class Potentiostat(serial.Serial):
     def set_device_id(self,device_id):
         """Sets the current value of the device identification number.
 
+        Args:
+            device_id (int): desired value for device identification number
+
+        Returns:
+            int: device identification number (from device).
+
         """
         cmd_dict = {CommandKey: SetDeviceIdCmd, DeviceIdKey: device_id}
         msg_dict = self.send_cmd(cmd_dict)
@@ -339,8 +409,14 @@ class Potentiostat(serial.Serial):
 
 
     def set_sample_period(self,sample_period):
-        """Sets the sample period (s) used for measurements. The sample period is the
-        time between samples. 
+        """Sets the sample period (ms) used for measurements. The sample period
+        is the time between samples. 
+
+        Args:
+            sample_period (int): desired sample period, units (ms)
+
+        Returns:
+            int: sample period (from device), units (ms)
 
         """
         cmd_dict = {CommandKey: SetSamplePeriodCmd, SamplePeriodKey: sample_period}
@@ -349,8 +425,11 @@ class Potentiostat(serial.Serial):
 
 
     def get_sample_period(self):
-        """Gets the current value for the sample period (s). The sample period is the
-        time between samples.
+        """Gets the current value for the sample period (ms). The sample period
+        is the time between samples.
+
+        Returns:
+            int: sample period (from device), units (ms)
 
         """
         cmd_dict = {CommandKey: GetSamplePeriodCmd}
@@ -359,8 +438,14 @@ class Potentiostat(serial.Serial):
 
 
     def set_sample_rate(self,sample_rate):
-        """Sets the measurement sample rate (Hz). Note, this is an alternative way to set 
-        the sample period. 
+        """Sets the measurement sample rate (Hz). Note, this is an alternative
+        way to set the sample period. 
+
+        Args:
+            sample_rate (float): desired sample rate, units (Hz)
+
+        Returns:
+            float: sample rate (from device), unit (Hz)
 
         """
         sample_period = int(1.0e3/sample_rate)
@@ -368,7 +453,11 @@ class Potentiostat(serial.Serial):
 
 
     def get_sample_rate(self):
-        """Gets the measurement sample period (Hz). Note, the sample rate is 1/sample_period.
+        """Gets the measurement sample period (Hz). Note, the sample rate is
+        1/sample_period.
+
+        Returns:
+            float: sample rate (from device), unit (Hz)
 
         """
         sample_period = self.get_sample_period()
@@ -377,7 +466,18 @@ class Potentiostat(serial.Serial):
 
 
     def get_test_done_time(self, test, timeunit='ms'):
-        """Gets the time in seconds required to complete the specified test including any quietTime, etc. 
+        """Gets the time in seconds required to complete the specified test
+        including any quietTime, etc. 
+
+        Args:
+            test (str): test name, e.g., 'cyclic'
+
+        Keyword Args:
+            timeunit (str): units to use for time, 'ms' (default) or 's'
+
+        Returns:
+            int/float: time required for test,  int if timeunit=='ms', float if timeunit=='s'
+
 
         """
         cmd_dict = {CommandKey: GetTestDoneTimeCmd, TestKey: test}
@@ -386,7 +486,11 @@ class Potentiostat(serial.Serial):
 
 
     def get_test_names(self):
-        """Gets the list of the names of all tests which can be performed by the device with the current firmware. 
+        """Gets the list of the names of all tests which can be performed by
+        the device with the current firmware. 
+
+        Returns:
+            list: names of all available tests. 
 
         """
         cmd_dict = {CommandKey: GetTestNamesCmd}
@@ -397,6 +501,9 @@ class Potentiostat(serial.Serial):
     def get_firmware_version(self):
         """Gets the version string for the firmware on the device.
 
+        Returns:
+            str: firmware version, e.g. 'FW0.0.8'
+
         """
         cmd_dict = {CommandKey: GetVersionCmd}
         msg_dict = self.send_cmd(cmd_dict)
@@ -405,6 +512,10 @@ class Potentiostat(serial.Serial):
 
     def get_hardware_version(self):
         """ Returns string representing the hardware version.
+
+        Returns:
+            str: hardware version, e.g., 'V0.2'
+
         """
         cmd_dict = {CommandKey: GetHardwareVersionCmd}
         try:
@@ -418,6 +529,13 @@ class Potentiostat(serial.Serial):
     def set_ref_elect_connected(self,value):
         """Sets the connected state (True/False) of the reference electrode.  This
         feature requires hardware version >= HW0.2
+
+        Args:
+            value (bool): desired connected state of reference electrode
+
+        Returns:
+            bool: connected state of the reference electrode (from device), True/False
+
         """
         self.check_hardware_version(HardwareVersion0p2)
         cmd_dict = {CommandKey: SetRefElectConnCmd, ConnectedKey: value}
@@ -426,8 +544,12 @@ class Potentiostat(serial.Serial):
 
 
     def get_ref_elect_connected(self):
-        """Gets the connected state (True/False) of the reference electrode.  This
-        feature requires hardware version >= HW0.2
+        """Gets the connected state (True/False) of the reference electrode.
+        This feature requires hardware version >= HW0.2
+
+        Returns:
+            bool: connected state of the reference electrode (from device), True/False
+
         """
         self.check_hardware_version(HardwareVersion0p2)
         cmd_dict = {CommandKey: GetRefElectConnCmd}
@@ -436,8 +558,15 @@ class Potentiostat(serial.Serial):
 
 
     def set_ctr_elect_connected(self,value):
-        """Sets the connected state (True/False) of the counter electrode.  This
-        feature requires hardware version >= HW0.2
+        """Sets the connected state (True/False) of the counter electrode.
+        This feature requires hardware version >= HW0.2
+
+        Args:
+            value (bool): desired connection state for counter electrode, True/False
+
+        Returns: 
+            bool: connected state of counter electrode (from device), True/False
+
         """
         self.check_hardware_version(HardwareVersion0p2)
         cmd_dict = {CommandKey: SetCtrElectConnCmd, ConnectedKey: value}
@@ -446,8 +575,12 @@ class Potentiostat(serial.Serial):
 
 
     def get_ctr_elect_connected(self):
-        """Gets the connected state (True/False) of the counter electrode.  This
-        feature requires hardware version >= HW0.2
+        """Gets the connected state (True/False) of the counter electrode.
+        This feature requires hardware version >= HW0.2
+
+        Returns:
+            bool: connection state of counter electrode (from device), True/False
+
         """
         self.check_hardware_version(HardwareVersion0p2)
         cmd_dict = {CommandKey: GetCtrElectConnCmd}
@@ -458,6 +591,12 @@ class Potentiostat(serial.Serial):
     def set_wrk_elect_connected(self,value):
         """Sets the connected state (True/False) of the working electrode.  This
         feature requires hardware version >= HW0.2
+
+        Args:
+            value (bool): desired connection state for working electrode, True/False
+
+        Returns:
+            bool: connection state of working electrode (from device), True/False
         """
         self.check_hardware_version(HardwareVersion0p2)
         cmd_dict = {CommandKey: SetWrkElectConnCmd, ConnectedKey: value}
@@ -468,6 +607,9 @@ class Potentiostat(serial.Serial):
     def get_wrk_elect_connected(self):
         """Gets the connected state (True/False) of the working electrode.  This
         feature requires hardware version >= HW0.2
+
+        Returns:
+            bool: connection state of working electrode (from device), True/False
         """
         self.check_hardware_version(HardwareVersion0p2)
         cmd_dict = {CommandKey: GetWrkElectConnCmd}
@@ -476,8 +618,16 @@ class Potentiostat(serial.Serial):
 
 
     def set_all_elect_connected(self,value):
-        """Sets the connected state (True/False) of all the electrodes (referene, 
-        counter and working). This feature requires hardware version >= HW0.2
+        """Sets the connected state (True/False) of all the electrodes
+        (referene, counter and working). This feature requires hardware version
+        >= HW0.2 
+
+        Args:
+            value (bool): desired connection state of all electrodes.
+
+        Returns:
+            bool: connection state of all electrodes (from device), True/False
+
         """
         self.check_hardware_version(HardwareVersion0p2)
         cmd_dict = {CommandKey: SetAllElectConnCmd, ConnectedKey: value}
@@ -489,6 +639,10 @@ class Potentiostat(serial.Serial):
         """Gets the connected state (True/False) of all the electrodes
         (referene, counter and working). Only True if all are connected. This
         feature requires hardware version >= HW0.2
+
+        Returns:
+            bool: connected stat of all electrodes (from device), True/False
+
         """
         self.check_hardware_version(HardwareVersion0p2)
         cmd_dict = {CommandKey: GetAllElectConnCmd}
@@ -498,9 +652,16 @@ class Potentiostat(serial.Serial):
 
     def set_auto_connect(self,value):
         """ Set auto-connect/auto-disconnect feature. If auto-connect is set to
-        True then the  ref, ctr and wrk electrodes will be automatically connected at
-        the beginning of each test and automatically disconnected at the end of
-        each test.
+        True then the  ref, ctr and wrk electrodes will be automatically
+        connected at the beginning of each test and automatically disconnected
+        at the end of each test.
+
+        Args:
+            value (bool): desired value for autoconnect, True/False
+
+        Returns:
+            bool: autoconnect state (from device), True/False
+
         """
         self.check_hardware_version(HardwareVersion0p2)
         cmd_dict = {CommandKey: SetElectAutoConnCmd, AutoConnectKey: value}
@@ -510,6 +671,10 @@ class Potentiostat(serial.Serial):
 
     def get_auto_connect(self):
         """ Gets the value of the device's auto-connect/auto-disconnect setting.
+
+        Returns:
+            bool: autoconnect state (from device), True/False
+
         """
         self.check_hardware_version(HardwareVersion0p2)
         cmd_dict = {CommandKey: GetElectAutoConnCmd}
@@ -519,6 +684,13 @@ class Potentiostat(serial.Serial):
 
     def set_ref_elect_volt_range(self,volt_range):
         """ Sets the voltage range for the reference electrode analog input.
+
+        Args:
+            volt_range (str): desired voltage range, e.g., '5V'
+
+        Returns:
+            str: reference electrode voltage range returnd by device
+
         """
         self.check_hardware_version(HardwareVersion0p2)
         if not volt_range in self.get_all_volt_range():
@@ -531,6 +703,10 @@ class Potentiostat(serial.Serial):
     def get_ref_elect_volt_range(self):
         """ Gets the  device's voltage range setting for the reference
         electrode analog input.  
+
+        Returns:
+            str: reference electrode voltage range returnd by device
+
         """
         self.check_hardware_version(HardwareVersion0p2)
         cmd_dict = {CommandKey: GetRefElectVoltRangeCmd}
@@ -541,6 +717,12 @@ class Potentiostat(serial.Serial):
     def set_mux_enabled(self, value):
         """Enable/Disables the multiplexer expansion hardware
 
+        Args:
+            value (bool): desired enable state of mux hardware, True/False
+
+        Returns:
+            bool: enable state of mux hardware (from device), True/Fasle
+
         """
         cmd_dict = {CommandKey: SetMuxEnabledCmd, MuxEnabledKey: value}
         msg_dict = self.send_cmd(cmd_dict)
@@ -550,6 +732,9 @@ class Potentiostat(serial.Serial):
     def get_mux_enabled(self):
         """Get multiplexer expansion hardware enabled state 
 
+        Returns:
+            bool: enable state of mux hardware (from device), True/Fasle
+
         """
         cmd_dict = {CommandKey: GetMuxEnabledCmd}
         msg_dict = self.send_cmd(cmd_dict)
@@ -557,7 +742,14 @@ class Potentiostat(serial.Serial):
 
 
     def set_enabled_mux_channels(self,channels):
-        """Enables the specified subset of multiplexer working electrode channels
+        """Enables the specified subset of multiplexer working electrode
+        channels
+
+        Args:
+            channels (list): list of mux channels to enable, e.g., [1,2,5]
+        
+        Returns:
+            list: enabled mux channels (from device).
 
         """
         cmd_dict = {CommandKey: SetEnabledMuxChanCmd, MuxChannelKey: channels}
@@ -566,7 +758,11 @@ class Potentiostat(serial.Serial):
 
 
     def get_enabled_mux_channels(self):
-        """Get the list of currently enabled multiplexer working electrode channels
+        """Get the list of currently enabled multiplexer working electrode
+        channels
+
+        Returns:
+            list: enabled mux channels (from device).
 
         """
         cmd_dict = {CommandKey: GetEnabledMuxChanCmd}
@@ -574,7 +770,11 @@ class Potentiostat(serial.Serial):
         return msg_dict[ResponseKey][MuxChannelKey]
 
     def get_mux_test_names(self):
-        """Gets the list of test which are compatible with the multiplexer expansion hardware
+        """Gets the list of tests which are compatible with the multiplexer
+        expansion hardware
+
+        Returns:
+            list: tests compatible with multiplexer 
 
         """
         cmd_dict = {CommandKey: GetMuxTestNamesCmd}
@@ -583,8 +783,14 @@ class Potentiostat(serial.Serial):
 
 
     def set_mux_ref_elect_connected(self,value):
-        """Sets the connected state (True/False) of the reference electrode when using the multiplexer 
-        expansion hardware.
+        """Sets the connected state (True/False) of the reference electrode
+        when using the multiplexer expansion hardware.
+
+        Args:
+            value (bool): desired connection for mux reference electrode, True/False
+
+        Returns:
+            bool: connection state of mux reference electrode (from device), True/False
 
         """
         cmd_dict = {CommandKey: SetMuxRefElectConnCmd, ConnectedKey: value}
@@ -593,8 +799,11 @@ class Potentiostat(serial.Serial):
 
 
     def get_mux_ref_elect_connected(self):
-        """ Returns the connected state (True/False) of the reference electrode when using the multiplexer
-        expansion hardware.
+        """ Gets the connected state (True/False) of the reference electrode
+        when using the multiplexer expansion hardware.
+
+        Returns:
+            bool: connection state of mux reference electrode (from device), True/False
 
         """
         cmd_dict = {CommandKey: GetMuxRefElectConnCmd}
@@ -603,8 +812,14 @@ class Potentiostat(serial.Serial):
 
 
     def set_mux_ctr_elect_connected(self,value):
-        """Sets the connected state (True or False) of the counter electrode when using the multiplexer
-        expansion hardware.
+        """Sets the connected state (True or False) of the counter electrode
+        when using the multiplexer expansion hardware.
+
+        Args:
+            value (bool): desired connection state for mux counter electrode, True/Fasle
+
+        Returns:
+            bool: connection state of mux counter electrode (from device), True/False
 
         """
         cmd_dict = {CommandKey: SetMuxCtrElectConnCmd, ConnectedKey: value}
@@ -613,8 +828,11 @@ class Potentiostat(serial.Serial):
 
 
     def get_mux_ctr_elect_connected(self):
-        """Returns the connected state (True or False) of the counter electrode when using the multiplexer
-        expansion hardware.
+        """Returns the connected state (True or False) of the counter electrode
+        when using the multiplexer expansion hardware.
+
+        Returns:
+            bool: connection state of mux counter electrode (from device), True/False
 
         """
         cmd_dict = {CommandKey: GetMuxCtrElectConnCmd}
@@ -623,9 +841,14 @@ class Potentiostat(serial.Serial):
 
 
     def set_mux_wrk_elect_connected(self, value):
-        """Sets the connected state (1, 2, 3, 4, 5, 6, 7 or False) of the working electrode when using the 
-        multiplexer expansion hardware.
+        """Sets the connected state (1, 2, 3, 4, 5, 6, 7 or False) of the
+        working electrode when using the multiplexer expansion hardware.
 
+        Args:
+            value (int/bool): desired connection state for mux working electrode.
+
+        Returns:
+            int/bool: connection state of mux working electrode.
         """
         cmd_dict = {CommandKey: SetMuxWrkElectConnCmd, ConnectedKey: value}
         msg_dict = self.send_cmd(cmd_dict)
@@ -633,9 +856,11 @@ class Potentiostat(serial.Serial):
 
 
     def get_mux_wrk_elect_connected(self):
-        """Returns the connected state (1, 2, 3, 4, 5, 6, 7 or False) of the working electrode when using 
-        the multiplexer expansion hardware.
+        """Returns the connected state (1, 2, 3, 4, 5, 6, 7 or False) of the
+        working electrode when using the multiplexer expansion hardware.
 
+        Returns:
+            int/bool: connection state of mux working electrode.
         """
         cmd_dict = {CommandKey: GetMuxWrkElectConnCmd}
         msg_dict = self.send_cmd(cmd_dict)
@@ -643,16 +868,50 @@ class Potentiostat(serial.Serial):
 
 
     def disconnect_all_mux_elect(self):
-        """Disconnects all electrodes (reference, counter and working) when using the multiplexer expansion
-        hardware.
-
+        """Disconnects all electrodes (reference, counter and working) when
+        using the multiplexer expansion hardware.
         """
         cmd_dict = {CommandKey: DisconnAllMuxElectCmd}
         msg_dict = self.send_cmd(cmd_dict)
 
 
-    def run_test(self, testname, param=None, filename=None, display='pbar', timeunit='s'):
-        """Runs the test with specified test name and returns the time, voltage and current data.
+    def run_test(self, testname, 
+            param=None, filename=None, on_data=None, display='pbar', timeunit='s'):
+        """Runs the test with specified test name and returns the time, voltage
+        and current data.
+
+        Args:
+            testname (str): name of test to run, e.g., 'cyclic', 'constant', linearSweep'
+
+        Keyword Args:
+
+            param   (dict): parameter settings for test, the default is None in
+                which case the current values are used.
+
+            filename (str): name of output file for saving data
+
+            on_data (function): callback function called on arrival of new data from 
+                the device. This function takes four arguments: chan, t, volt, and curr. 
+
+            display  (str): output display mode, display=='pbar' show a progressbar,  
+                display=='data' shows a text stream and display=='' shows nothing 
+
+            timeunit (str): units for time values, if timeunit=='s' then time
+                values are in seconds otherwise if timeunit=='ms' then time values
+                are is milliseconds.
+
+        Returns:
+            tuple/dict: data acquired during test. 
+            
+            When the multiplexer is disabled a tuple (t,volt,curr) of lists is
+            returned containing the measurement times (s), voltages (V) and
+            currents (uA) respectively
+
+            When the multiplexer is enabled run_test returns a dictionary
+            containing the measured data for each channel. The keys of the
+            dictionary returned are the enabled channel numbers and the values
+            are dictionaries with the time, voltage and current measurments for
+            that channel, keyed by ‘t’, ‘v’, and ‘i’ respectively.
 
         """
         mux_enabled = False
@@ -703,7 +962,7 @@ class Potentiostat(serial.Serial):
         done = False
         while not done:
 
-            # Get dat from device
+            # Get data from device
             sample_json = self.readline()
             sample_json = sample_json.strip()
             sample_dict = json.loads(sample_json.decode())
@@ -733,9 +992,12 @@ class Potentiostat(serial.Serial):
                         print('{0:1.3f}, {1:1.4f}, {2:1.4f}'.format(tval,volt,curr))
                     else:
                         print('ch{0}: {1:1.3f}, {2:1.4f}, {3:1.4f}'.format(chan,tval,volt,curr))
-
                 elif display == 'pbar':
                     pbar.update(tval)
+
+                # Pass data to callback function if present
+                if on_data is not None:
+                    on_data(chan, tval, volt, curr)
             else:
                 done = True
 
@@ -765,6 +1027,15 @@ class Potentiostat(serial.Serial):
         """Sends a command to the device.  Low-level method - command is specified 
         using command dictionary.
 
+        Args:
+            cmd_dict (dict): commands to be sent to the device, e.g., {'command': 'getVersion'}
+
+        Returns:
+            dict: JSON response message from device converted to a dictionary. 
+
+            For example, the response to {'command': 'getVersion'} might be
+            {'success': True, 'response': {'command': 'getVersion', 'version': 'FW0.0.8'}}
+
         """
         cmd_json = json.dumps(cmd_dict) + '\n'
         self.write(cmd_json.encode())
@@ -777,6 +1048,16 @@ class Potentiostat(serial.Serial):
 
 
     def check_cmd_msg(self,cmd_dict,msg_dict):
+        """
+        Checks the msg response to a given command. 
+
+        :meta private:
+
+        Args:
+            cmd_dict (dict): the command sent to the device
+            msg_dict (dict): the response returned by the device
+
+        """
         self.check_for_success(msg_dict)
         self.check_cmd_match(cmd_dict,msg_dict)
         if TestKey in cmd_dict:
@@ -784,6 +1065,15 @@ class Potentiostat(serial.Serial):
 
 
     def check_for_success(self,msg_dict):
+        """
+        Check the response message to determine whether or not is was successful. 
+
+        :meta private:
+
+        Args:
+            msg_dict (dict): the response returned by the device. 
+
+        """
         try: 
             success = msg_dict[SuccessKey] 
         except KeyError:
@@ -794,6 +1084,16 @@ class Potentiostat(serial.Serial):
 
 
     def check_cmd_match(self,cmd_dict,msg_dict):
+        """
+        Check to verify that command and response match.
+
+        :meta private:
+
+        Args:
+            cmd_dict (dict): the command sent to the device
+            msg_dict (dict): the response returned by the device
+            
+        """
         cmd_sent = cmd_dict[CommandKey]
         cmd_recv = msg_dict[ResponseKey][CommandKey]
         if cmd_recv != cmd_sent: 
@@ -801,6 +1101,16 @@ class Potentiostat(serial.Serial):
 
 
     def check_test_match(self,cmd_dict,msg_dict): 
+        """
+        Check to verify that the test name sent matches that recieved. 
+
+        :meta private:
+
+        Args:
+            cmd_dict (dict): the command sent to the device
+            msg_dict (dict): the response returned by the device
+
+        """
         test_sent = cmd_dict[TestKey]
         test_recv = msg_dict[ResponseKey][TestKey]
         if test_recv != test_sent:
@@ -808,13 +1118,23 @@ class Potentiostat(serial.Serial):
 
 
     def check_hardware_version(self,hw_version):
-        """ Check to see if the device's hardware version is at least 'hw_version'
+        """ Check to see if the device's hardware version matches the given
+        string 'hw_version'
+
+        :meta private:
+
+        Args: 
+            hw_version (str): string representing hardware version to check, e.g., 'HW0.2'
+
         """
         if self.hardware_version < hw_version:
             raise RuntimeError('requires hardware version >= {}'.format(HardwareVersion0p2))
 
 
     def atexit_cleanup(self):
+        """
+        :meta private:
+        """
         if self.isOpen() and self.test_running:
             self.stop_test()
 
